@@ -1,6 +1,6 @@
 package YATT::Lite::Test::XHFTest;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use parent qw(YATT::Lite::Object);
 use YATT::Lite::MFields qw/tests numtests yatt global file_list file_dict
 	      cf_filename cf_ext cf_parser cf_encoding
@@ -37,10 +37,12 @@ use Encode;
 		cf_PARAM
 		cf_OUT
 		cf_ERROR
+                cf_ERROR_BODY
 
 		cf_REQUIRE
 
 		cf_TAG
+                cf_CON_CLASS
 	      /;
 
   sub is_runnable { shift->ntests }
